#
#  This is the Plone Help Center FAQ Folder type, with enhanced features
#  like dividing the FAQ into Sections, and Display relevant
#  versions.
#

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchemaFolderish, HelpCenterContainerSchema
from PHCFolder import PHCFolder

FAQFolderSchema = HelpCenterBaseSchemaFolderish + Schema((
    TextField('description',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                description="Description of the FAQ Container.",
                description_msgid="phc_desc_folder",
                label_msgid="phc_label_folder",
                label="Description",
                i18n_domain = "plonehelpcenter",
                rows=6,)
        ),
    ),) + HelpCenterContainerSchema

class HelpCenterFAQFolder(PHCFolder,OrderedBaseFolder):
    """An FAQ Section can hold frequently asked questions with answers."""

    __implements__ = (PHCFolder.__implements__,
        OrderedBaseFolder.__implements__)

    content_icon = 'faq_icon.gif'

    schema = FAQFolderSchema
    archetype_name = 'FAQ Section'
    meta_type = 'HelpCenterFAQFolder'
    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ('HelpCenterFAQ', )

    security = ClassSecurityInfo()

    typeDescription= 'An FAQ Section can hold frequently asked questions with answers.'
    typeDescMsgId  = 'description_edit_faqfolder'

    # aliases = PHCFolder.aliases.copy()
    # aliases.update({'(Default)' : 'faqfolder_view',
    #                 'view'      : 'faqfolder_view'})

registerType(HelpCenterFAQFolder, PROJECTNAME)



from Testing import ZopeTestCase

# Make the boring stuff load quietly
ZopeTestCase.installProduct('CMFCore', quiet=1)
ZopeTestCase.installProduct('CMFDefault', quiet=1)
ZopeTestCase.installProduct('CMFCalendar', quiet=1)
ZopeTestCase.installProduct('CMFTopic', quiet=1)
ZopeTestCase.installProduct('DCWorkflow', quiet=1)
ZopeTestCase.installProduct('CMFActionIcons', quiet=1)
ZopeTestCase.installProduct('CMFQuickInstallerTool', quiet=1)
ZopeTestCase.installProduct('CMFFormController', quiet=1)
ZopeTestCase.installProduct('GroupUserFolder', quiet=1)
ZopeTestCase.installProduct('ZCTextIndex', quiet=1)
ZopeTestCase.installProduct('TextIndexNG2', quiet=1)
ZopeTestCase.installProduct('SecureMailHost', quiet=1)
ZopeTestCase.installProduct('CMFPlone')

ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PortalTransforms', quiet=1)
ZopeTestCase.installProduct('MimetypesRegistry', quiet=1)
ZopeTestCase.installProduct('PloneHelpCenter')

from Products.PloneTestCase import PloneTestCase
from Products.PloneHelpCenter.config import ADD_CENTER_PERMISSION, \
  ADD_HELP_AREA_PERMISSION

PRODUCTS = ['Archetypes', 'PloneHelpCenter']

PloneTestCase.setupPloneSite(products=PRODUCTS)


class PHCTestCase(PloneTestCase.PloneTestCase):

    defaultTitle = 'Default Testing Title'
    defaultVersions = ( 'Version 1.0', 'Version 2.0', 'Different Version1.0', )
    defaultBodyRst = """
    Bogus reST body
    ===============
    
    Here's fake body content for unit tests.
    
    * Looks like a list.
    * Smells like a list.
    * It's a list!
    
    Final content after the list.
    """

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()

    def afterSetUp(self):
        self.portal.manage_permission(ADD_CENTER_PERMISSION,
                                      ['Manager', 'Owner'])
        self.portal.manage_permission(ADD_HELP_AREA_PERMISSION,
                                      ['Manager', 'Owner'])
        self._createHelpCenter(self.folder)

    def _createHelpCenter(self, folder, id='hc', title=defaultTitle, versions=defaultVersions):
        """Creates and returns a refence to a PHC HelpCenter.
        This method publishes a HelpCenter instance under folder.  It fills in
        all of the standard properties."""
        folder.invokeFactory('HelpCenter', id)
        helpCenter = getattr(folder, id)
        helpCenter.setTitle(title)
        helpCenter.setDescription('A HelpCenter instance for unit tests.')
        helpCenter.setVersionsVocab(versions)
        self.portal.portal_workflow.doActionFor(helpCenter, 'submit')
        return helpCenter

    def _createHowto(self, howtoFolder, id, title=defaultTitle):
        """Creates and returns a refence to a PHC Howto.
        This method creates a Howto instance under a folder.  It fills in
        all of the standard properties."""
        howtoFolder.invokeFactory('HelpCenterHowTo', id)
        howto = getattr(howtoFolder, id)
        howto.setTitle(title)
        howto.setDescription('A PHC Howto for unit tests.')
        howto.setBody(self.defaultBodyRst) 
        howto.setVersions( ('Version 2.0',) )
        howto.setSections( ('General',) )
        self.portal.plone_utils.editMetadata(howto, format='text/x-rst')
        return howto

    def _createTutorial(self, tutorialFolder, id, title=defaultTitle, numPages=2):
        """Creates and returns a reference to a PHC Tutorial.
        This method creates a Tutorial instance under a folder.  It fills in
        all of the standard properties."""
        tutorialFolder.invokeFactory('HelpCenterTutorial', id)
        tutorial = getattr(tutorialFolder, id)
        tutorial.setTitle(title)
        tutorial.setDescription('A PHC Tutorial for unit tests.')
        tutorial.setVersions( ('Version 2.0',) )
        tutorial.setSections( ('General',) )
        # attach pages
        for i in range(numPages):
            pageNum = i + 1
            id='page%d' % pageNum
            tutorial.invokeFactory('HelpCenterTutorialPage', id)
            newPage = getattr(tutorial, id)
            newPage.setTitle('Test Tutorial Page %d' % pageNum)
            newPage.setDescription('A PHC Tutorial Page (%d) for unit tests.' % pageNum)
            newPage.setBody(self.defaultBodyRst)
            self.portal.plone_utils.editMetadata(newPage,format='text/x-rst')
        return tutorial
                                   
    def _createFAQ(self, faqFolder, id, title=defaultTitle):
        """Creates and returns a reference to a PHC FAQ.
        This method creates an FAQ instance under a folder.  It fills in
        all of the standard properties."""
        faqFolder.invokeFactory('HelpCenterFAQ', id=id)
        faq = getattr(faqFolder, id)
        faq.setTitle=(title)
        faq.setDescription('An FAQ for unit tests.  Did you know that this field is supposed to be the questionfaq.set?')
        faq.setAnswer('No one knows; it is one of the great mysteries.')
        faq.setVersions( ('Version 2.0',) )
        faq.setSections( ('General',) )
        self.portal.plone_utils.editMetadata(faq, format='text/plain')
        return faq

    def _createLink(self, linkFolder, id, title=defaultTitle):
        linkFolder.invokeFactory('HelpCenterLink', id)
        link = getattr(linkFolder, id)
        link.setTitle( title )
        link.setDescription( 'A Link for unit tests.' )
        link.setUrl('http://www.plone.org/')
        link.setVersions( ('Version 2.0',) )
        link.setSections( ('General',) )
        return link

    def _createErrorReference(self, errorRefFolder, id, title=defaultTitle):
        errorRefFolder.invokeFactory('HelpCenterErrorReference', id)
        errorRef = getattr(errorRefFolder, id)
        errorRef.setTitle( title )
        errorRef.setDescription( 'An error reference for unit tests.' )
        errorRef.setBody( self.defaultBodyRst )
        errorRef.setVersions( ('Version 2.0',) )
        errorRef.setSections( ('General',) )
        self.portal.plone_utils.editMetadata(errorRef, format='text/x-rst')
        return errorRef

    def _createDefinition(self, glossaryFolder, id, title=defaultTitle):
        glossaryFolder.invokeFactory('HelpCenterDefinition', id)
        definition = getattr(glossaryFolder, id)
        definition.setTitle(title)
        definition.setDescription('A definition for unit tests.')
        definition.setVersions( ('Version 2.0',) )
        definition.setSections( ('General',) )
        return definition

    def _createReferenceManual(self, glossaryFolder, id, title=defaultTitle):
        pass

    def _createVideo(self, videoFolder, id, title=defaultTitle):
        pass

class PHCFunctionalTestCase(PloneTestCase.FunctionalTestCase):
    
    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.FunctionalTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()




try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchema, GenericHelpCenterItemSchema
from PHCContent import PHCContent

DefinitionSchema = HelpCenterBaseSchema + Schema((
    TextField(
        'description',
        default='',
        searchable=1,
        required=1,
        primary=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=AttributeStorage(), # Needed for HistoryAwareMixin
        widget = TextAreaWidget(
                    description = 'An explanation of the term.',
                    description_msgid = "phc_desc_definition",
                    label = "Definition",
                    label_msgid = "phc_label_definition",
                    rows = 5,
                    i18n_domain = "plonehelpcenter",
                    )
        ),
    ),) + GenericHelpCenterItemSchema

# For some reason, we need to jump through these hoops to get the fields in the
# the right order
DefinitionSchema.moveField('subject', pos='bottom')
DefinitionSchema.moveField('relatedItems', pos='bottom')

class HelpCenterDefinition(PHCContent,BaseContent):
    """A Definition defines a special term, and will be listed in the glossary.
    """

    __implements__ = (PHCContent.__implements__,
                      BaseContent.__implements__,)

    content_icon = 'glossary_icon.gif'

    schema = DefinitionSchema
    archetype_name = 'Definition'
    meta_type = 'HelpCenterDefinition'
    global_allow = 0
    # allow_discussion = IS_DISCUSSABLE

    typeDescription= 'A Definition defines a special term, and will be listed in the glossary.'
    typeDescMsgId  = 'description_edit_definition'

    # aliases = PHCContent.aliases.copy()
    # aliases.update({'(Default)' : 'definition_view',
    #                 'view'      : 'definition_view'})

registerType(HelpCenterDefinition, PROJECTNAME)



try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
from Products.PloneHelpCenter.config import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from schemata import HelpCenterBaseSchema, HelpCenterItemSchema
from PHCContent import PHCContent
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.CMFPlone.browser.navtree import NavtreeStrategyBase, buildFolderTree

import re
IMG_PATTERN = re.compile(r"""(\<img .*?)src="([^/]+?)"(.*?\>)""", re.IGNORECASE | re.DOTALL)


ReferenceManualSchema = HelpCenterBaseSchema + Schema((
    TextField(
        'description',
        default='',
        searchable=1,
        required=1,
        primary=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                description='A summary of the reference manual -- subject and scope. Will be displayed on every page of the manual.',
                description_msgid="phc_help_referencemanual_summary",
                label="Reference Manual Description",
                label_msgid="phc_label_referencemanual_description",
                rows=5,
                i18n_domain="plonehelpcenter",
                ),
        ),
    ),)  + HelpCenterItemSchema

# For some reason, we need to jump through these hoops to get the fields in the
# the right order
ReferenceManualSchema.moveField('subject', pos='bottom')
ReferenceManualSchema.moveField('relatedItems', pos='bottom')


class HelpCenterReferenceManual(PHCContent,OrderedBaseFolder):
    """A reference manual containing ReferenceManualPages,
    ReferenceManualSections, Files and Images.
    """

    __implements__ =(PHCContent.__implements__,
                      OrderedBaseFolder.__implements__,)

    schema = ReferenceManualSchema
    archetype_name = 'Reference Manual'
    meta_type='HelpCenterReferenceManual'
    content_icon = 'referencemanual_icon.gif'

    global_allow = 0
    filter_content_types = 1
    allowed_content_types =('HelpCenterReferenceManualPage', 
                             'HelpCenterReferenceManualSection', 
                             'Image', 'File')
    # allow_discussion = IS_DISCUSSABLE

    security = ClassSecurityInfo()

    typeDescription= 'A Reference Manual can contain Reference Manual Pages and Sections, Images and Files. Index order is decided by the folder order, use the normal up/down arrow in the folder content view to rearrange content.'
    typeDescMsgId  = 'description_edit_referencemanual'

    security.declareProtected(CMFCorePermissions.View,
                                'getReferenceManualDescription')
    def getReferenceManualDescription(self):
        """ Returns the description of the ReferenceManual -- 
        convenience method for ReferenceManualPage
        """
        return self.Description()

    security.declareProtected(CMFCorePermissions.View, 'getTOC')
    def getTOC(self, current=None, root=None):
        """Get the table-of-contents of this manual. 
        
        The parameter 'current' gives the object that is the current page or
        section being viewed. 
        
        The parameter 'root' gives the root of the manual - if not given, this
        ReferenceManual object is used, but you can pass in a 
        ReferenceManualSection instead to root the TOC at this element. The 
        root element itself is not included in the table-of-contents.
        
        The return value is a list of dicts, recursively representing the 
        table-of-contents of this manual. Each element dict contains:
        
            item        -- a catalog brain for the item (a section or page)
            numbering   -- The dotted numbering of this item, e.g. 1.3.2
            depth       -- The depth of the item (0 == top-level item)
            currentItem -- True if this item corresponds to the object 'current'
            children    -- A list of dicts
            
        The list 'children' recursively contains the equivalent dicts for
        children of each section. If the parameter 'current' is not given, no
        element will have current == True.
        """

        if not root:
            root = self

        class Strategy(NavtreeStrategyBase):
            
            rootPath = '/'.join(root.getPhysicalPath())
            showAllParents = False
                
        strategy = Strategy()
        query=  {'path'        : '/'.join(root.getPhysicalPath()),
                 'portal_type' : ('HelpCenterReferenceManualSection',
                                   'HelpCenterReferenceManualPage',),
                 'sort_on'     : 'getObjPositionInParent'}
                
        toc = buildFolderTree(self, current, query, strategy)['children']
        
        def buildNumbering(nodes, base=""):
            idx = 1
            for n in nodes:
                numbering = "%s%d." % (base, idx,)
                n['numbering'] = numbering
                buildNumbering(n['children'], numbering)
                idx += 1
                
        buildNumbering(toc)
        return toc

    security.declareProtected(CMFCorePermissions.View, 'getTOCInfo')
    def getTOCInfo(self, toc):
        """Get information about a table-of-contents, as returned by getTOC.
        
        The return value is a dict, containing:

            tocList    -- A flat list representing the table-of-contents
            localTOC   -- A toc structure for the contents under the current
                            item (passed in as 'current' to getTOC())
            currentIdx -- The index in tocList of 'current'
            nextIdx    -- The index in tocList of the next item after 'current'
            prevIdx    -- The index in tocList of the next item before 'current'
            parentIdx  -- The index in tocList of the parent of 'current'
            
        The elements 'currentIdx', 'nextIdx', 'prevIdx' and 'parentIdx' may be
        None if either the table-of-contents was not constructed with a current
        item, or if there is no previous/next/parent item. Similarly, 'localTOC' 
        will be None if the table-of-contents was not constructed with a current 
        item.
        
        Each item in the list 'tocList' in the returned dict contains a dict 
        with keys:
        
            item       -- A catalog brain repsenting the item
            numbering  -- The dotted numbering of the item, e.g. 1.3.2.
            depth      -- The depth of the item (0 = top-level item)
            current    -- True if this item represents the current page/section
         
        The parameter 'toc' gives the table of contents, as returned by
        getTOC() above.
        """

        # Let's fake static variables - keep track of what may be our parent
        global parentIdx, prevIdx, prevDepth, prevWasCurrent

        parentIdx = None
        prevIdx = None
        prevDepth = -1
        prevWasCurrent = False

        def addToList(tocInfo, tocItem):
            item      = tocItem['item']
            numbering = tocItem['numbering']
            depth     = tocItem['depth']
            children  = tocItem['children']
            isCurrent = tocItem['currentItem']

            global parentIdx, prevIdx, prevDepth, prevWasCurrent

            numberingList = numbering.split('.')[:-1]
            idxList = [int(number) - 1 for number in numberingList]

            tocInfo['tocList'].append({'item'        : item,
                                       'numbering'   : numbering,
                                       'depth'       : depth,
                                       'currentItem' : isCurrent,
                                       })

            currentIdx = len(tocInfo['tocList']) - 1

            if isCurrent:
                prevWasCurrent = True
                tocInfo['currentIdx'] = currentIdx
                if currentIdx > 0:
                    tocInfo['prevIdx'] = currentIdx - 1
                tocInfo['parentIdx'] = parentIdx
                tocInfo['localTOC'] = tocItem['children']
            elif prevWasCurrent:
                prevWasCurrent = False
                tocInfo['nextIdx'] = currentIdx

            for child in children:
                addToList(tocInfo, child)

            # parent index will be item with depth = depth - 1
            # keep track of potential parents by noting when we move down
            # one step

            if depth > prevDepth:
                parentIdx = prevIdx
                prevDepth = depth

            prevIdx = currentIdx

        tocInfo = {'currentIdx' : None,
                   'nextIdx'    : None,
                   'prevIdx'    : None,
                   'parentIdx'  : None,
                   'tocList'    : [],
                   'localTOC'   : None}

        for topLevel in toc:
            addToList(tocInfo, topLevel)

        return tocInfo

    security.declareProtected(CMFCorePermissions.View, 'addImagePaths')
    def addImagePaths(self, body, baseurl):
        """Fixup image paths in section body"""
        
        # This is a convenience method for use in referencemanual_macros
        # section_collation macro. It looks in body for img tags
        # with relative URLs in the src attribute and prepends the baseurl.
        # TODO: when we not longer need 2.1 compatibility, this belongs in 
        # a view.
                
        return IMG_PATTERN.sub(r"""\1src="%s/\2"\3""" % baseurl, body)


registerType(HelpCenterReferenceManual, PROJECTNAME)




#!/usr/bin/env python

"""
   Usage: filter.py <target-product> <source-product>

   Filter out all msgid's in target product that are already in source product.

   Note that PYTHON and I18NDUDE must have been set as enviroment variables before calling this script
"""

import os, sys

__PYTHON = os.environ.get('PYTHON', '')
__I18NDUDE = os.environ.get('I18NDUDE', '')

def main():
    if len(sys.argv) < 3:
        print 'You have to specify the target and source product.'
        sys.exit(1)

    target = sys.argv[1]+'.pot'
    source = sys.argv[2]+'.pot'

    os.chdir('..')

    if not os.path.isfile(source):
        print 'Source pot was not found for the given product.'
        sys.exit(2)

    if not os.path.isfile(target):
        print 'Target pot was not found for the given product.'
        sys.exit(3)

    os.system(__PYTHON + ' ' + __I18NDUDE + (' filter %s %s > %s-new') % (target, source, target))

if __name__ == '__main__':
    main()



try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchema
from PHCContent import PHCContent

TutorialPageSchema = HelpCenterBaseSchema + Schema((
    TextField(
        'description',
        default='',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                  description="Enter a brief description.",
                  description_msgid="phc_desc_tutorial_page",
                  label="Description",
                  label_msgid="phc_label_tutorial_page",
                  rows=5,
                  i18n_domain="plonehelpcenter",
                  ),
        ),

    TextField(
        'body',
        required=1,
        searchable=1,
        primary=1,
        widget=RichWidget(
                description="The body text.",
                description_msgid="phc_desc_body_tutorial",
                label="Body text",
                label_msgid="phc_label_body_tutorial",
                rows=25,
                i18n_domain="plonehelpcenter"
                ),
        **DEFAULT_CONTENT_TYPES
        ),
    ),)

class HelpCenterTutorialPage(PHCContent,BaseContent):
    """Part of a tutorial."""

    __implements__ = (PHCContent.__implements__,
                      BaseContent.__implements__,)

    schema = TutorialPageSchema
    archetype_name = 'Page'
    meta_type='HelpCenterTutorialPage'
    content_icon = 'document_icon.gif'

    global_allow = 0
    # allow_discussion = 1

    typeDescription= 'A Tutorial Page contains the text of a single page of the tutorial.'
    typeDescMsgId  = 'description_edit_tutorialpage'


registerType(HelpCenterTutorialPage, PROJECTNAME)



# 
#
# Generated by dumpDCWorkflow.py written by Sebastien Bigaret
# Original workflow id/title: helpcenter_workflow/Default Workflow [Plone]
# Date: 2004/06/06 04:31:31.859 GMT-4
#
# WARNING: this dumps does NOT contain any scripts you might have added to
# the workflow, IT IS YOUR RESPONSABILITY TO MAKE BACKUPS FOR THESE SCRIPTS.
#
# No script detected in this workflow
# 
"""
Programmatically creates a workflow type
"""
__version__ = "$Revision: 1.8 $"[11:-2]

from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.PloneHelpCenter.config import *

def setupHelpcenter_workflow(wf):
    "..."
    wf.setProperties(title='Default Workflow [PloneHelpCenter]')

    for s in ['in-progress', 'obsolete', 'pending', 'published']:
        wf.states.addState(s)
    for t in ['retract', 'mark_obsolete', 'submit', 'publish', 'reject', 're-activate']:
        wf.transitions.addTransition(t)
    for v in ['action', 'review_history', 'actor', 'comments', 'time']:
        wf.variables.addVariable(v)
    for l in ['reviewer_queue']:
        wf.worklists.addWorklist(l)
    for p in ('Access contents information', 'Modify portal content', 'View', 
                'Change portal events', 'List folder contents', 'Add portal content',):
        wf.addManagedPermission(p)
        

    ## Initial State
    wf.states.setInitialState('in-progress')

    ## States initialization
    sdef = wf.states['in-progress']
    sdef.setProperties(title="""In progress""",
                       transitions=('mark_obsolete', 'publish', 'submit'))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 0, ['Manager', 'Owner'])
    sdef.setPermission('List folder contents', 0, ['Manager', 'Owner'])
    sdef.setPermission('Change portal events', 0, ['Manager', 'Owner'])
    sdef.setPermission('Add portal content', 0, ['Manager', 'Owner'])

    sdef = wf.states['obsolete']
    sdef.setProperties(title="""Obsolete""",
                       transitions=('re-activate',))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('View', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('List folder contents', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Change portal events', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Add portal content', 0, ['Manager', 'Owner', 'Reviewer'])

    sdef = wf.states['pending']
    sdef.setProperties(title="""Waiting for reviewer""",
                       transitions=('mark_obsolete', 'publish', 'reject', 'retract'))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('View', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('List folder contents', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Change portal events', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('Add portal content', 0, ['Manager', 'Owner', 'Reviewer'])

    sdef = wf.states['published']
    sdef.setProperties(title="""Published""",
                       transitions=('mark_obsolete', 'reject', 'retract'))
    sdef.setPermission('Access contents information', 1, ['Anonymous', 'Manager'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 1, ['Anonymous', 'Manager'])
    sdef.setPermission('List folder contents', 0, ['Anonymous', 'Manager'])
    sdef.setPermission('Change portal events', 0, ['Manager'])
    sdef.setPermission('Add portal content', 0, ['Manager', 'Owner'])


    ## Transitions initialization
    tdef = wf.transitions['retract']
    tdef.setProperties(title="""Member retracts submission""",
                       new_state_id="""in-progress""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Retract""",
                       actbox_url="""%(content_url)s/content_retract_form""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'Request review'},
                       )

    tdef = wf.transitions['mark_obsolete']
    tdef.setProperties(title="""Mark Obsolete""",
                       new_state_id="""obsolete""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Mark obsolete""",
                       actbox_url="""%(content_url)s/content_hide_form""",
                       actbox_category="""workflow""",
                       props={'guard_expr': 'python: user.has_permission("Review portal content", here) or user.has_role("Owner", here)' },
                       )

    tdef = wf.transitions['submit']
    tdef.setProperties(title="""Member requests publishing""",
                       new_state_id="""pending""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Submit""",
                       actbox_url="""%(content_url)s/content_submit_form""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'Request review'},
                       )

    tdef = wf.transitions['publish']
    tdef.setProperties(title="""Reviewer publishes content""",
                       new_state_id="""published""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Publish""",
                       actbox_url="""%(content_url)s/content_publish_form""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'Review portal content'},
                       )

    tdef = wf.transitions['reject']
    tdef.setProperties(title="""Reviewer rejects submission""",
                       new_state_id="""in-progress""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Reject""",
                       actbox_url="""%(content_url)s/content_reject_form""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'Review portal content'},
                       )
                       
    tdef = wf.transitions['re-activate']
    tdef.setProperties(title="""Re-activates obsolete item""",
                       new_state_id="""in-progress""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Re-activate""",
                       actbox_url="""%(content_url)s/content_retract_form""",
                       actbox_category="""workflow""",
                       props={'guard_expr': 'python: user.has_permission("Review portal content", here) or user.has_role("Owner", here)' },
                       )

    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    vdef = wf.variables['action']
    vdef.setProperties(description="""The last transition""",
                       default_value="""""",
                       default_expr="""transition/getId|nothing""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['review_history']
    vdef.setProperties(description="""Provides access to workflow history""",
                       default_value="""""",
                       default_expr="""state_change/getHistory""",
                       for_catalog=0,
                       for_status=0,
                       update_always=0,
                       props={'guard_permissions': 'Request review; Review portal content'})

    vdef = wf.variables['actor']
    vdef.setProperties(description="""The ID of the user who performed the last transition""",
                       default_value="""""",
                       default_expr="""user/getId""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['comments']
    vdef.setProperties(description="""Comments about the last transition""",
                       default_value="""""",
                       default_expr="""python:state_change.kwargs.get('comment', '')""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['time']
    vdef.setProperties(description="""Time of the last transition""",
                       default_value="""""",
                       default_expr="""state_change/getDateTime""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    ## Worklists Initialization
    ldef = wf.worklists['reviewer_queue']
    ldef.setProperties(description="""Reviewer tasks""",
                       actbox_name="""Pending (%(count)d)""",
                       actbox_url="""%(portal_url)s/search?review_state=pending""",
                       actbox_category="""global""",
                       props={'guard_permissions': 'Review portal content', 'var_match_review_state': 'pending'})


def createHelpcenter_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupHelpcenter_workflow(ob)
    return ob

def install():
    addWorkflowFactory(createHelpcenter_workflow,
                       id='helpcenter_workflow',
                       title='Workflow for Help Center Items.')



try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
from Products.PloneHelpCenter.config import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from schemata import HelpCenterBaseSchema, HelpCenterItemSchema
from PHCContent import PHCContent
from AccessControl import ClassSecurityInfo

TutorialSchema = HelpCenterBaseSchema + Schema((
    TextField(
        'description',
        default='',
        searchable=1,
        required=1,
        primary=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                 description = 'A summary of the tutorial--aims and scope. Will be displayed on every page of the tutorial.',
                 description_msgid = "phc_help_tutorial_summary",
                 label = "Tutorial Description",
                 label_msgid = "phc_label_tutorial_description",
                 rows = 5,
                 i18n_domain = "plonehelpcenter",
                )
        ),
    ),)  + HelpCenterItemSchema

# For some reason, we need to jump through these hoops to get the fields in the
# the right order
TutorialSchema.moveField('subject', pos='bottom')
TutorialSchema.moveField('relatedItems', pos='bottom')

class HelpCenterTutorial(PHCContent,OrderedBaseFolder):
    """A tutorial containing TutorialPages, Files and Images."""

    __implements__ = (PHCContent.__implements__,
                      OrderedBaseFolder.__implements__,)

    schema = TutorialSchema
    archetype_name = 'Tutorial'
    meta_type = portal_type = 'HelpCenterTutorial'
    content_icon = 'tutorial_icon.gif'

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ('HelpCenterTutorialPage', 'Image', 'File')
    # allow_discussion = IS_DISCUSSABLE

    typeDescription= 'A Tutorial can contain Tutorial Pages, Images and Files. Index order is decided by the folder order, use the normal up/down arrow in the folder content view to rearrange content.'
    typeDescMsgId  = 'description_edit_tutorial'

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.View, 'getTutorialDescription')
    def getTutorialDescription(self):
        """ Returns the description of the Tutorial--convenience method for TutorialPage """
        return self.Description()

    security.declareProtected(CMFCorePermissions.View, 'getPages')
    def getPages(self, states=[]):
        """Get items"""
        criteria = contentFilter = {'portal_type' : 'HelpCenterTutorialPage'}
        if states:
            criteria['review_state'] = states
        return self.getFolderContents(contentFilter = criteria)

    security.declareProtected(CMFCorePermissions.View, 'getPagePosition')
    def getPagePosition(self, obj, states=[]):
        """Get position in folder of the current context"""
        pages = self.getPages()
        for i in range(len(pages)):
            if pages[i].getId == obj.getId():
                return i
        return None

registerType(HelpCenterTutorial, PROJECTNAME)




try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchema, GenericHelpCenterItemSchema
from PHCContent import PHCContent

FAQSchema = HelpCenterBaseSchema + Schema((
    TextField(
        'description',
        default='',
        searchable=1,
        required=0,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                description = 'More details on the question, if not evident from the title.',
                description_msgid = "help_detailed_question",
                label = "Detailed Question",
                label_msgid = "label_detailed_question",
                rows = 5,
                i18n_domain = "plonehelpcenter",
                ),
        ),
        
    TextField(
        'answer',
        required=1,
        searchable=1,
        primary=1,
        widget=RichWidget(
                label_msgid = "label_answer",
                i18n_domain = "plonehelpcenter",
                rows=10),
        **DEFAULT_CONTENT_TYPES
        ),
    ),) + GenericHelpCenterItemSchema

# For some reason, we need to jump through these hoops to get the fields in the
# the right order
FAQSchema.moveField('subject', pos='bottom')
FAQSchema.moveField('relatedItems', pos='bottom')

class HelpCenterFAQ(PHCContent,BaseContent):
    """A Frequently Asked Question defines a common question with an answer - 
    this is a place to document answers to common questions, not ask them.
    """

    __implements__ = (PHCContent.__implements__,
                      BaseContent.__implements__,)

    content_icon = 'faq_icon.gif'

    schema = FAQSchema
    archetype_name = 'FAQ'
    meta_type = 'HelpCenterFAQ'
    global_allow = 0
    # allow_discussion = IS_DISCUSSABLE

    typeDescription= 'A Frequently Asked Question defines a common question with an answer - this is a place to document answers to common questions, not ask them.'
    typeDescMsgId  = 'description_edit_faq'

    # aliases = PHCContent.aliases.copy()
    # aliases.update({'(Default)' : 'faq_view',
    #                 'view'      : 'faq_view'})


registerType(HelpCenterFAQ, PROJECTNAME)



try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.PloneHelpCenter.config import *

try:
    from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget
    PHCReferenceWidget = ReferenceBrowserWidget
except ImportError:
    PHCReferenceWidget = ReferenceWidget

try:
    from Products.AddRemoveWidget import AddRemoveWidget
    PHCKeywordWidget = AddRemoveWidget
except ImportError:
    PHCKeywordWidget = KeywordWidget


###########################################
# Common components to Help Types schemas #
###########################################

HelpCenterItemSchema = Schema((

    LinesField(
        'versions',
        languageIndependent=1,
        index='KeywordIndex:brains',
        vocabulary='getVersionsVocab',
        condition='object/getVersionsVocab',
        multiValued=1,
        required=0,
        widget=MultiSelectionWidget(
                label_msgid='phc_label_versions',
                label= "Versions",
                condition='object/getVersionsVocab',
                description='Versions of product that apply to this item ' \
                            '(leave blank if not version-specific).',
                description_msgid = "phc_versions",
                i18n_domain = "plonehelpcenter"
                ),
        ),
    
    LinesField(
        'sections',
        multiValued=1,
        required=0,
        vocabulary='getSectionsVocab',
        condition='object/getSectionsVocab',
        index='KeywordIndex:schema',
        widget=MultiSelectionWidget(
                label='Sections',
                condition='object/getSectionsVocab',
                description='Section(s) that this item should appear in.',
                description_msgid = "phc_sections",
                label_msgid = "phc_label_sections",
                i18n_domain = "plonehelpcenter",
                ),
        ),
    
    LinesField(
        'audiences',
        multiValued=1,
        required=0,
        vocabulary='getAudiencesVocab',
        condition="object/getAudiencesVocab",
        index='KeywordIndex:schema',
        widget=MultiSelectionWidget(
                label='Audiences',
                description='Audience(s) this item is targetted at.',
                description_msgid = "phc_audiences",
                condition="object/getAudiencesVocab",
                label_msgid = "phc_label_audiences",
                i18n_domain = "plonehelpcenter",
                ),
        ),
    
    LinesField(
        'contributors',
        accessor="Contributors",
        languageIndependent=1,
        widget=LinesWidget(
                label='Contributors',
                label_msgid="label_contributors",
                description="Enter additional names (no need to include the current owner) for those who have contributed to this entry, one per line.",
                description_msgid="help_contributors",
                i18n_domain="plone",
                ),
        ),
    
    LinesField(
        'subject',
        accessor='Subject',
        searchable=1,
        vocabulary='getSubjectVocab',
        enforceVocabulary=0,
        isMetadata=1,
        widget=PHCKeywordWidget(
                label='Related keywords',
                label_msgid = "phc_label_related",
                i18n_domain="plonehelpcenter",
        ),
    ),

    BooleanField(
        'startHere',
        index='FieldIndex:schema',
        permission = CMFCorePermissions.ReviewPortalContent,
        widget=BooleanWidget(
                label='Start Here',
                description="Marks this as a good starting point for its section. Only key documents should have this property.",
                description_msgid = "phc_starthere",
                label_msgid = "phc_label_starthere",
                i18n_domain="plonehelpcenter"
        ),
    ),
    
    ReferenceField(
        'relatedItems',
        relationship='PloneHelpCenter',
        allowed_types=REFERENCEABLE_TYPES,
        required = 0,
        multiValued=1,
        languageIndependent=1,
        widget=PHCReferenceWidget (
                label="Referenced Items",
                description="Set one or more references to HelpCenter items.",
                description_msgid = "phc_reference",
                label_msgid = "phc_label_reference",
                i18n_domain="plonehelpcenter"
                ),
    ),
    
))

GenericHelpCenterItemSchema = HelpCenterItemSchema.copy()
del GenericHelpCenterItemSchema['audiences']

# what sections should there be? (for enclosing folders, not indiv items!)
HelpCenterContainerSchema = Schema((

    LinesField(
        'sectionsVocab',
        accessor='getSectionsVocab',
        edit_accessor='getRawSectionsVocab',
        mutator='setSectionsVocab',
        widget=LinesWidget(
                   label="Sections",
                   description="One section on each line. Used for grouping items. If you leave this blank, the help center's sections will be used. If both are blank, sections will not be used.",
                   description_msgid = "phc_sections_vocab",
                   label_msgid = "phc_label_sections-vocab",
                   i18n_domain="plonehelpcenter",
                   rows=6,
                   )
        ),
    ))

# non folderish Help Center Base schemata
HelpCenterBaseSchema = BaseSchema.copy()

# folderish Help Center Base schemata
HelpCenterBaseSchemaFolderish = BaseFolderSchema.copy()


# Remove "contributors" from metadata, so that we can add it later
if GLOBAL_RIGHTS:
    del HelpCenterBaseSchema['contributors']
    del HelpCenterBaseSchemaFolderish['contributors']

    del HelpCenterBaseSchema['rights']
    del HelpCenterBaseSchemaFolderish['rights']



#
# The Plone How-to container.
#
# The main goals of these containers are to restrict the addable types and
# provide a sensible default view out-of-the-box, like the FAQ view.
#


try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchemaFolderish, HelpCenterContainerSchema
from PHCFolder import PHCFolder

ErrorReferenceFolderSchema = HelpCenterBaseSchemaFolderish + Schema((
    TextField('description',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
            description_msgid="description_edit_ErrorReferenceFolder",
            description="Description for the Error Reference section.",
            label_msgid="description_label_ErrorReferenceFolder",
            label="Description",
            i18n_domain = "plonehelpcenter",
            rows=6)
        ),
    ),) + HelpCenterContainerSchema

class HelpCenterErrorReferenceFolder(PHCFolder,BaseFolder):
    """An Error Reference Section can contain references to and explanations of 
    common errors.
    """

    __implements__ = (PHCFolder.__implements__,
        OrderedBaseFolder.__implements__)

    content_icon = 'errorref_icon.gif'

    schema = ErrorReferenceFolderSchema
    archetype_name = 'Error Reference Section'
    meta_type = 'HelpCenterErrorReferenceFolder'
    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ('HelpCenterErrorReference', )

    typeDescription= 'An Error Reference Section can contain references to and explanations of common errors.'
    typeDescMsgId  = 'description_edit_errorreferencefolder'

    security = ClassSecurityInfo()

    # aliases = PHCFolder.aliases.copy()
    # aliases.update({'(Default)' : 'errorreferencefolder_view',
    #                 'view'      : 'errorreferencefolder_view'})

registerType(HelpCenterErrorReferenceFolder, PROJECTNAME)



"""\
PloneHelpCenter tests package

To run all tests type 'python runalltests.py'
"""

# For TTW functional testing, it's nice to create some data
# that hangs around after the tests are done.  Uncomment the
# next three lines to tie some test data generation scripts
# to the portal_migration tool's Setup tab.  (Yeah, it's an
# abuse of portal_migration.  We need a portal_test_script tool.)
#
#from Products.CMFPlone import MigrationTool
#from Products.PloneHelpCenter.tests.CustomSetup import CustomSetup
#MigrationTool.registerSetupWidget(CustomSetup)



import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.i18ntestcase import PotTestCase, PoTestCase
from Products.i18ntestcase.I18NTestCase import getPoFiles, getPotFiles, getProductFromPath
from i18ndude import catalog
from Globals import package_home

GLOBALS = globals()
PACKAGE_HOME = os.path.normpath(os.path.join(package_home(GLOBALS), '..'))

head, tail = os.path.split(PACKAGE_HOME)
if tail == 'tests':
    PACKAGE_HOME = os.path.join(PACKAGE_HOME, '..')

i18ndir = os.path.normpath(PACKAGE_HOME)

tests=[]
products=[]
pots={}
pot_catalogs={}
pot_lens={}

for potFile in getPotFiles(path=i18ndir):
    product = getProductFromPath(potFile)
    if product not in products:
        products.append(product)
    if product not in pot_catalogs:
        cat = catalog.MessageCatalog(filename=potFile)
        cat_len = len(cat)
        pots.update({product: potFile})
        pot_catalogs.update({product: cat})
        pot_lens.update({product: cat_len})

for product in products:
    class TestOnePOT(PotTestCase.PotTestCase):
        product = product
        pot = pots[product]
    tests.append(TestOnePOT)

    for poFile in getPoFiles(path=i18ndir, product=product):
        class TestOnePoFile(PoTestCase.PoTestCase):
            po = poFile
            product = product
            pot_cat = pot_catalogs[product]
            pot_len = pot_lens[product]
        tests.append(TestOnePoFile)

    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        for test in tests:
            suite.addTest(unittest.makeSuite(test))
        return suite

if __name__ == '__main__':
    framework()




#!/usr/bin/env python

"""
   Usage: chart.py <product>
   Note that PYTHON and I18NDUDE must have been set as enviroment variables before calling this script
"""

import os, sys
from utils import getPoFilesAsCmdLine, getLongProductName

__PYTHON = os.environ.get('PYTHON', '')
__I18NDUDE = os.environ.get('I18NDUDE', '')

def main():
    if len(sys.argv) == 1:
        print 'You have to specify the product.'
        sys.exit(1)

    product = getLongProductName(sys.argv[1])
    pot = '%s.pot' % product
    chart = os.curdir + os.sep + 'charts' + os.sep + '%s-chart.gif' % product

    os.chdir('..')

    if not os.path.isfile(pot):
        print 'No pot was found for the given product.'
        sys.exit(2)

    poFiles = getPoFilesAsCmdLine(product)
    if poFiles == '':
        print 'No po-files were found for the given product.'
        sys.exit(3)

    os.system(__PYTHON + ' ' + __I18NDUDE + (' chart -o %s --pot %s %s') % (chart, pot, poFiles))

if __name__ == '__main__':
    main()



""" support for HelpCenter templates """

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.PythonScripts.standard import url_quote_plus

from Products.PloneHelpCenter.config import TOPIC_VIEW_TYPES


subtypes_tuples = (
    ('HelpCenterFAQFolder','HelpCenterFAQ'),
    ('HelpCenterHowToFolder','HelpCenterHowTo'),
    ('HelpCenterTutorialFolder','HelpCenterTutorial'),
    ('HelpCenterLinkFolder','HelpCenterLink'),
    ('HelpCenterErrorReferenceFolder','HelpCenterErrorReference'),
    ('HelpCenterGlossary','HelpCenterDefinition'),
    ('HelpCenterReferenceManualFolder','HelpCenterReferenceManual'),
    ('HelpCenterInstructionalVideoFolder','HelpCenterInstructionalVideo')
    )


# case-insensitive compare
def ncCmp(a, b):
    return cmp(a.lower(), b.lower())

# compare items by startHere and caseless title
def itemCmp(a, b):
    ash = getattr(a, 'getStartHere', False)
    bsh = getattr(b, 'getStartHere', False)
    if (ash and bsh) or (not ash and not bsh):
        return cmp(a.Title.lower(), b.Title.lower())
    elif ash:
        return -1
    else:
        return 1


class HelpCenterView(BrowserView):
    """ support for HelpCenter templates """

    def __init__(self, context, request):
        """ set up a few convenience object attributes """
        
        BrowserView.__init__(self, context, request)

        self.catalog = getToolByName(self.context, 'portal_catalog')
        self.portal_url = getToolByName(self.context, 'portal_url')()      
        self.context_path = '/'.join(self.context.getPhysicalPath())


    def subtypes(self):
        """ returns a list of major container types """
        return [t[0] for t in subtypes_tuples]
    

    def rss_subtypes(self):
        """ returns a list of doc types """
        return [t[1] for t in subtypes_tuples]
        

    def getSyndicationURL(self):
        """ returns a URL for RSS feed of help doc types """

        return self.portal_url + '/search_rss?sort_on=modified&sort_order=descending&path=' \
         + self.context_path + '&' + ('&'.join(['portal_type=%s' % s[1] for s in subtypes_tuples ]))

    
    def sections(self):
        """ subtype sections in current folder """

        contentFilter = {'review_state':('published', 'visible',), 'portal_type' : self.subtypes()}
        return self.context.getFolderContents(contentFilter=contentFilter)


    def sectionContents(self, section, limit=5):
        """ return section contents """
        
        contentFilter = {'review_state':'published','sort_on':'modified','sort_order':'reverse', 'limit' : limit}
        return section.getObject().getFolderContents(contentFilter=contentFilter);        


    def getTopics(self):
        """Returns list of major topics and subtopics; used in helpcenter_topicview"""
        
        # major topics are those defined in the HelpCenter object.
        # returns list in the form:
        # [{title, url, subtopics}, ...]
        # subtopics are [{title, url}, ...]
        
        here_url  = self.context.absolute_url()

        # get a set of the major topics
        try:
            majorTopics = self.context.getSectionsVocab()
        except KeyError:
            return []
        liveSections = set( self.catalog.uniqueValuesFor('getSections') )
        
        sections = []
        currTitle = ''
        for live in majorTopics:
            if live in liveSections:
                # break into main topic, sub topic
                cs = [s.strip() for s in live.split(':')]
                main = cs[0]
                sub = ': '.join(cs[1:])
                # sub = cs[-1]
                
                if main != currTitle:
                    # append a new topic dict
                    currTitle = main
                    currSubSections = []
                    sections.append(
                     {'title':currTitle,
                      'subtopics':currSubSections, 
                      'url': here_url + '/phc_topic_area?topic=' + url_quote_plus(currTitle),
                      }
                     )
                if sub:
                    # add to the subtopics list
                    id = sub.lower().replace(' ','-')  # make HTML anchor ID
                    currSubSections.append(
                     {'title':sub,
                      'url': "%s/phc_topic_area?topic=%s#%s" % (here_url, url_quote_plus(currTitle), id)
                      }
                     )

        #sections.sort(indexCmp)        
        return sections


    def getSectionMap(self):
        """
          returns a complex list of section dicts
          [{title:sectiontitle, subtopics:listOfSubTopics, url:urlOfSection, count:itemsInSection}, ...]
          subtopics are each lists [{title:titleOfSubSection,url:urlOfSubSection}]
          This is used in helpcenter_ploneorg.pt.
        """
        
        context = self.context
        
        here_url  = context.absolute_url()
        phc = context.getPHCObject()
        
        topics = phc.getSectionsVocab()
        
        # dict to save counts
        topicsDict = {}
        
        for topic in topics:
            if ':' not in topic:
                items = self.catalog(portal_type=['HelpCenterReferenceManual','HelpCenterTutorial','HelpCenterHowTo'],
                                               review_state='published',
                                               getSections=[topic])
                for item in items:
                    for section in item.getSections:
                        topicsDict[section] = topicsDict.get(section, 0) + 1
        
        sections = []
        currTitle = ''
        for topic in topics:
            count = topicsDict.get(topic)
            if count:
                # break into main topic, sub topic
                cs = [s.strip() for s in topic.split(':')]
                main = cs[0]
                sub = ': '.join(cs[1:])
                
                if main != currTitle:
                    # append a new topic dict
                    currTitle = main
                    currSubSections = []
                    sections.append(
                     {'title':currTitle,
                      'subtopics':currSubSections,
                      'url': here_url + '/phc_topic_area?topic=' + url_quote_plus(currTitle),
                      'count':count
                      }
                     )
                if sub:
                    # add to the subtopics list
                    id = sub.lower().replace(' ','-')  # make HTML anchor ID
                    currSubSections.append(
                     {'title':sub,
                      'url': "%s/phc_topic_area?topic=%s#%s" % (here_url, url_quote_plus(currTitle), id)
                      }
                     )
            
        return sections


    def getSubTopics(self, topic="Visual Design", portal_types=TOPIC_VIEW_TYPES):
        """Get subtopics for phc_topic_area -- a utility for the phc_topicarea view"""
        
        # Returns sorted list of dicts in the form:
        # { 'title': title, 'id':id, 'docs': docs, }
        # docs are a sorted list of document brains

        # get a list of brains for all items of matching type and topic
        items = self.catalog(portal_type=portal_types, 
                             getSections=topic, 
                             path=self.context.getPHCPath())
        
        # construct a dict of subtopics under this topic
        # with a list of matching items as value
        subtopics = {}
        for item in items:
            # Add item to subtopics by section.
            # Sections should have the format "topic : subtopic"
            sections = []
            for s in item.getSections:
                if s.startswith(topic):
                    pos = s.find(':')
                    if pos > 0:
                        sections.append(s[pos+1:].strip())
            if sections:
                for s in sections:
                    subtopics.setdefault(s, []).append(item)
            else:
                subtopics.setdefault('!!!General', []).append(item)
            
        # Sort the subtopics
        keys = subtopics.keys()
        keys.sort(ncCmp)
        
        # organize into an array of dicts with keys: title, id, docs
        # where docs is a sorted list of docs in section with 'start here' docs at top
        sorted_list = []
        for k in keys:
          title = k.replace('!!!','') # strip off force-alpha
          id = title.lower().replace(' ','-')  # make HTML anchor ID
          docs = subtopics[k]
          docs.sort(itemCmp)
          sorted_list.append( { 'title': title, 'id':id, 'docs': docs, } )
        
        return sorted_list


    def getMajorTopics(self):
        """Returns a sorted list of major sections"""

        topics = {}
        for topic in self.context.getSectionsVocab():
            pos = topic.find(':')
            if pos > 0:
                topics.setdefault(topic[:pos].strip(), 1)
            else:
                topics.setdefault(topic, 1)
        
        keys = topics.keys()
        keys.sort(ncCmp)
        
        return keys


    def getNonPHCContents(self):
        """Get a list of folder objects of types not defined by the help
        center product which have been placed in this help center.
        """
        
        phcTypes = self.context.allowed_content_types
        allTypes = self.catalog.uniqueValuesFor('portal_type')

        nonPHCTypes = [t for t in allTypes if t not in phcTypes]

        # Need this, else we end up with all types, not no types :)
        if not nonPHCTypes:
            return []

        return self.context.getFolderContents(contentFilter = {'portal_type' : nonPHCTypes}, batch=True)


    def statsQueryCatalog(self):
        """
            Find documentation items based on a statistics query.
            Used in phc_stats_search.cpt
        """
        
        REQUEST = self.context.REQUEST
        
        catalogMatches = self.context.queryCatalog(REQUEST=REQUEST, use_types_blacklist=False)
        matches = [m for m in catalogMatches]
        
        # Now do checks for comments, multiple sections/audiences/versions
        
        multipleSections = REQUEST.get('getSections_multiple', False)
        if multipleSections:
            matches = [m for m in matches if len(m.getSections) > 1]
            
        noSections = REQUEST.get('getSections_none', False)
        if noSections:
            matches = [m for m in matches if len(m.getSections) == 0]
            
        multipleAudiences = REQUEST.get('getAudiences_multiple', False)
        if multipleAudiences:
            matches = [m for m in matches if len(m.getAudiences) > 1]
        
        multipleVersions = REQUEST.get('getVersions_multiple', False)
        if multipleVersions:
            matches = [m for m in matches if len(m.getVersions) > 1]
            
        noVersions = REQUEST.get('getVersions_none', False)
        if noVersions:
            matches = [m for m in matches if len(m.getVersions) == 0]
            
        hasComments = REQUEST.get('hasComments', False)
        if hasComments:
            matchedPaths = [m.getPath() for m in matches]
            comments = self.catalog.searchResults(path = matchedPaths, 
                                                  portal_type = 'Discussion Item')
                                             
            foundPaths = {}
            for c in comments:
                path = c.getPath()
                idx = path.rfind('/talkback')
                path = path[:idx]
                id = path.split('/')[-1]
                foundPaths[path] = 1
                
            matches = [m for m in matches if m.getPath() in foundPaths]
                
        return matches

    def searchTypes(self):
        """
         Determine which portal_types to search
         based on the request and our type list.
        """
        
        # get our choice from the form request
        request = self.context.REQUEST
        choice = request.form.get("phc_selection",None)
        
        # map our sections to our eligible choices
        choiceTypes={
            "faq":('HelpCenterFAQ','HelpCenterFAQFolder'),
            "howto":("HelpCenterHowTo","HelpCenterHowtoFolder"),
            "tutorial":("HelpCenterTutorial","HelpCenterTutorialFolder", \
                        "HelpCenterTutorialPage"),
            "link":("HelpCenterLink","HelpCenterLinkFolder"),
            "error":("HelpCenterErrorReference","HelpCenterErrorReferenceFolder"),
            "glossary":("HelpCenterDefinition","HelpCenterGlossary"),
            "manual":("HelpCenterReferenceManual","HelpCenterReferenceManualFolder", \
                    "HelpCenterReferenceManualSection","HelpCenterReferenceManualPage"),
            "video":("HelpCenterInstructionalVideo","HelpCenterInstructionalVideoFolder"),
        }
        
        if choice and choiceTypes.has_key(choice):
            result = choiceTypes.get(choice)
        else:
            # choice must have been "all documentation"
            result=['HelpCenterFAQ',
                    'HelpCenterHowTo',
                    'HelpCenterTutorial',
                    'HelpCenterTutorialPage',
                    'HelpCenterLink',
                    'HelpCenterErrorReference',
                    'HelpCenterDefinition',
                    'HelpCenterReferenceManual',
                    'HelpCenterReferenceManualSection',
                    'HelpCenterReferenceManualPage',
                    'HelpCenterInstructionalVideo']
        
        return result



#
# Tests for Link types in the PHC
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneHelpCenter.tests import PHCTestCase


class TestLink(PHCTestCase.PHCTestCase):
    """General tests for Link Folder and Link objects."""

    def afterSetUp(self):
        PHCTestCase.PHCTestCase.afterSetUp(self)
        self._createLink(self.folder.hc.link, 'l')
        self.link = self.folder.hc.link.l

    def testInitialSections(self):
        # Test that the default section list is correct.
        self.assertEqual(self.link.getSections(), ('General',))

    def testVersionsonLink(self):
        versions = ('1.0','2.0','Strange version')
        self.folder.hc.setVersionsVocab(versions)
        newVersions = self.link.getVersionsVocab()
        self.assertEqual(newVersions, versions)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestLink))
    return suite

if __name__ == '__main__':
    framework()



from zope.interface import Interface

class IHelpCenterContent(Interface):
    """ The HelpCenter
    """

class IHelpCenterFolder(Interface):
    """ Marker interface for PHCFolder class that is
        the ancestory of the PHC containers.
    """

class IHelpCenterHowTo(Interface):
    """ Marker interface for HelpCenterHowTo class.
    """


try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
from Products.PloneHelpCenter.config import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from schemata import HelpCenterBaseSchema
from PHCContent import PHCContent
from AccessControl import ClassSecurityInfo

ReferenceManualSectionSchema = HelpCenterBaseSchema + Schema((
    TextField(
        'description',
        default='',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                description = "Enter a brief description for this section of the manual.",
                description_msgid = "phc_desc_referencemanual_section",
                label = "Description",
                label_msgid = "phc_label_referencemanual_section",
                rows = 5,
                i18n_domain = "plonehelpcenter",
                )
        ),
    ),)

class HelpCenterReferenceManualSection(PHCContent,OrderedBaseFolder):
    """A section of a reference manual containing ReferenceManualPages and
    other ReferenceManualSections.
    """

    __implements__ = (PHCContent.__implements__,
                      OrderedBaseFolder.__implements__,)

    schema = ReferenceManualSectionSchema
    archetype_name = 'Section'
    meta_type = 'HelpCenterReferenceManualSection'
    content_icon = 'chapter_icon.gif'

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ('HelpCenterReferenceManualPage', 'Image',
                                'HelpCenterReferenceManualSection')
    # allow_discussion = IS_DISCUSSABLE

    typeDescription= 'A Reference Manual Section can contain Reference Manual Pages, and other Reference Manual (Sub-)Sections. Index order is decided by the folder order, use the normal up/down arrow in the folder content view to rearrange content.'
    typeDescMsgId  = 'description_edit_referencemanualsection'

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.View, 'getSectionDescription')
    def getSectionDescription(self):
        """ Returns the description of the section --
        convenience method for ReferenceManualPage
        """
        return self.Description()

registerType(HelpCenterReferenceManualSection, PROJECTNAME)




try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchemaFolderish, GenericHelpCenterItemSchema
from PHCContent import PHCContent
from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder

ErrorReferenceSchema = HelpCenterBaseSchemaFolderish + Schema((

    TextField('description',
        default='',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget = TextAreaWidget(
                  description = "Enter a brief description.",
                  description_msgid = "phc_desc_ErrorReference",
                  label = "Description",
                  label_msgid = "phc_label_ErrorReference",
                  rows = 5,
                  i18n_domain = "plonehelpcenter",
                  ),
        ),
        
    TextField('body',
        searchable=1,
        required=1,
        primary=1,
        widget=RichWidget(description_msgid='phc_desc_body_ErrorReference',
            description='Explanation of the error.',
            label_msgid='phc_label_body_ErrorReference',
            label='Body',
            i18n_domain = "plonehelpcenter",
            rows=25,
            ),

        **DEFAULT_CONTENT_TYPES
        ),
    ),) + GenericHelpCenterItemSchema

# For some reason, we need to jump through these hoops to get the fields in the
# the right order
ErrorReferenceSchema.moveField('subject', pos='bottom')
ErrorReferenceSchema.moveField('relatedItems', pos='bottom')

class HelpCenterErrorReference(PHCContent,OrderedBaseFolder):
    """An Error Reference can be used to explain a particular error which may 
    arise.
    """ 

    __implements__ = (PHCContent.__implements__,
                      OrderedBaseFolder.__implements__,
                      INonStructuralFolder)


    content_icon = 'errorref_icon.gif'

    schema = ErrorReferenceSchema
    archetype_name = 'Error Reference'
    meta_type = 'HelpCenterErrorReference'
    global_allow = 0
    filter_content_types = 1
    # allow_discussion = IS_DISCUSSABLE
    allowed_content_types = ('Image', 'File',)

    typeDescription= 'An Error Reference can be used to explain a particular error which may arise.'
    typeDescMsgId  = 'description_edit_errorreference'

    actions = PHCContent.actions + (
        {
            'id': 'attachments',
            'name': 'Attachments',
            'action': 'string:${object_url}/attachments',
            'permissions': (CMFCorePermissions.ModifyPortalContent,)
        },
    )

    # aliases = PHCContent.aliases.copy()
    # aliases.update({'(Default)'   : 'errorreference_view',
    #                 'view'        : 'errorreference_view',
    #                 'attachments' : 'phc_attachments'})

    security = ClassSecurityInfo()

registerType(HelpCenterErrorReference, PROJECTNAME)



# 
#
# Generated by dumpDCWorkflow.py written by Sebastien Bigaret
# Original workflow id/title: helpcenter_workflow/Default Workflow [Plone]
# Date: 2004/06/06 04:31:31.859 GMT-4
#
# WARNING: this dumps does NOT contain any scripts you might have added to
# the workflow, IT IS YOUR RESPONSABILITY TO MAKE BACKUPS FOR THESE SCRIPTS.
#
# No script detected in this workflow
# 
"""
Programmatically creates a workflow type
"""
__version__ = "$Revision: 1.1 $"[11:-2]

from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.PloneHelpCenter.config import *

def setupHelpcentersubitem_workflow(wf):
    "..."
    wf.setProperties(title='Sub-item workflow [PloneHelpCenter]')

    for s in ['hidden', 'published']:
        wf.states.addState(s)
    for t in ['hide', 'publish']:
        wf.transitions.addTransition(t)
    for v in ['action', 'review_history', 'actor', 'comments', 'time']:
        wf.variables.addVariable(v)
    for l in ['reviewer_queue']:
        wf.worklists.addWorklist(l)
    for p in ('Access contents information', 'Modify portal content', 'View', 'Change portal events',):
        wf.addManagedPermission(p)
        

    ## Initial State
    wf.states.setInitialState('hidden')

    ## States initialization
    sdef = wf.states['hidden']
    sdef.setProperties(title="""Draft""",
                       transitions=('publish',))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('View', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Change portal events', 0, ['Manager', 'Owner', 'Reviewer'])

    sdef = wf.states['published']
    sdef.setProperties(title="""Public""",
                       transitions=('hide',))
    sdef.setPermission('Access contents information', 1, ['Manager'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 1, ['Manager'])
    sdef.setPermission('Change portal events', 0, ['Manager'])


    ## Transitions initialization
    tdef = wf.transitions['hide']
    tdef.setProperties(title="""Owner hides item from the public""",
                       new_state_id="""hidden""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Hide""",
                       actbox_url="""%(content_url)s/content_hide_form""",
                       actbox_category="""workflow""",
                       props={'guard_expr': 'python: user.has_permission("Review portal content", here) or user.has_role("Owner", here)' },
                       )

    tdef = wf.transitions['publish']
    tdef.setProperties(title="""Reviewer publishes content""",
                       new_state_id="""published""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Publish""",
                       actbox_url="""%(content_url)s/content_publish_form""",
                       actbox_category="""workflow""",
                       props={'guard_expr': 'python: user.has_permission("Review portal content", here) or user.has_role("Owner", here)' },
                       )

    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    vdef = wf.variables['action']
    vdef.setProperties(description="""The last transition""",
                       default_value="""""",
                       default_expr="""transition/getId|nothing""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['review_history']
    vdef.setProperties(description="""Provides access to workflow history""",
                       default_value="""""",
                       default_expr="""state_change/getHistory""",
                       for_catalog=0,
                       for_status=0,
                       update_always=0,
                       props={'guard_permissions': 'Request review; Review portal content'})

    vdef = wf.variables['actor']
    vdef.setProperties(description="""The ID of the user who performed the last transition""",
                       default_value="""""",
                       default_expr="""user/getId""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['comments']
    vdef.setProperties(description="""Comments about the last transition""",
                       default_value="""""",
                       default_expr="""python:state_change.kwargs.get('comment', '')""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['time']
    vdef.setProperties(description="""Time of the last transition""",
                       default_value="""""",
                       default_expr="""state_change/getDateTime""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    ## Worklists Initialization
    ldef = wf.worklists['reviewer_queue']
    ldef.setProperties(description="""Reviewer tasks""",
                       actbox_name="""Pending (%(count)d)""",
                       actbox_url="""%(portal_url)s/search?review_state=pending""",
                       actbox_category="""global""",
                       props={'guard_permissions': 'Review portal content', 'var_match_review_state': 'pending'})


def createHelpcentersubitem_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupHelpcentersubitem_workflow(ob)
    return ob

def install():
    addWorkflowFactory(createHelpcentersubitem_workflow,
                       id='helpcentersubitem_workflow',
                       title='Workflow for Help Center Sub-items.')



## Script (Python) ""
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters= 
##title=

return [(v.title, k) for k, v in context.portal_workflow['helpcenter_workflow'].states.items()]




try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo
from Products.PloneHelpCenter.config import *
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_hasattr, base_hasattr
from AccessControl import getSecurityManager
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from zope.interface import implements
from Products.PloneHelpCenter.interfaces import IHelpCenterFolder

# Compare section brains by title
def _sectionCmp(a, b):
    # depends on cmp(True, False) == 1
    ash = getattr(a, 'getStartHere', False)
    bsh = getattr(b, 'getStartHere', False)
    if ash == bsh:
        return cmp(a.Title.lower(), b.Title.lower())
    elif ash:
        return -1
    else:
        return 1


class PHCFolder(BrowserDefaultMixin):
    """A simple mixin for folderish archetype"""

    implements(IHelpCenterFolder)
    
    __implements__ = BrowserDefaultMixin.__implements__

    _at_rename_after_creation = True

    # empty right slot by default
    _properties = OrderedBaseFolder._properties +  (
        {'id'    : 'right_slots',
         'type'  : 'lines',
         'mode'  : 'rw',
         'label' : 'Right slots'
         },
         )

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.View, 'getAudiencesVocab')
    def getAudiencesVocab(self):
        """Get sections vocabulary."""
        if not safe_hasattr(self.aq_parent, 'getAudiencesVocab'):
            return []
        return self.aq_parent.getAudiencesVocab()


    security.declareProtected(CMFCorePermissions.View, 'getVersionsVocab')
    def getVersionsVocab(self):
        """Get version vocabulary."""
        if not safe_hasattr(self.aq_parent, 'getVersionsVocab'):
            return []
        return self.aq_parent.getVersionsVocab()
            

    security.declareProtected(CMFCorePermissions.View, 'getSectionsVocab')
    def getSectionsVocab(self):
        """Get sections vocabulary"""
        
        localSections = self.sectionsVocab
        if len(localSections):
            return localSections
        elif base_hasattr(self.aq_parent, 'getSectionsVocab'):
            return self.aq_parent.sectionsVocab
        else:
            return ()



## Script (Python) "fetchStaleItems"
##title=Query for "stale" HelpCenter items, i.e. unchanged for a specified interval
##parameters=stale_interval

# Construct a catalog query using the specified interval (count back from
# the current date-time) and return the list of matching result objects.
# This will populate the stale-HelpCenter-items portlet.
thresholdValues = {
    '1_day': 1,
    '1_week': 7,
    '2_weeks': 14,
    '1_month': 30,
    '3_months': 90,
    '6_months': 180,
    '1_year': 365,
}

stale_threshold_time = DateTime() - thresholdValues[ stale_interval ]

results = context.portal_catalog.searchResults(
    modified = { 'query': stale_threshold_time, 'range': 'max' },
    review_state=['published'], 
    portal_type=[
        'HelpCenterDefinition',
        'HelpCenterErrorReference',
        'HelpCenterFAQ',
        'HelpCenterHowTo',
        'HelpCenterLink',
        'HelpCenterTutorial',
    ],
    sort_on='portal_type',
)

return results




# These migrations were used during pre-release days. Hopefully, they should
# not be needed any more.

# This is used by the wiki-to-helpcenter migrator, which we only ran once
## from Products.ATContentTypes.migration.common import *
## from Products.ATContentTypes.migration.Walker import CatalogWalker
## from Products.ATContentTypes.migration.Migrator import BaseMigrator
## from Products.ATContentTypes.migration.Migrator import CMFItemMigrator

from Products.CMFCore.utils import getToolByName
from Acquisition import aq_parent, aq_base

from StringIO import StringIO

# This is used by the wiki-to-helpcenter migrator, which we only ran once
##BASE_PATH = '/testsite/documentation/howto'
##
##class DocumentationWalker(CatalogWalker):
##    """Overwrite walk to start from a base path
##    """
##    
##    def walk(self, **kwargs):
##        """Walks around and returns all objects which needs migration
##
##        :return: objects (with acquisition wrapper) that needs migration
##        :rtype: generator
##        """
##        LOG("fromType: " + str(self.fromType))
##        catalog = self.catalog
##
##        if HAS_LINGUA_PLONE and 'Language' in catalog.indexes():
##            # usage of Language is required for LinguaPlone
##            brains = catalog(portal_type = self.fromType,
##                             Language = catalog.uniqueValuesFor('Language'),
##                             path = BASE_PATH,
##                            )
##        else:
##            brains = catalog(portal_type = self.fromType,
##                             path = BASE_PATH,)
##
##        for brain in brains:
##            obj = brain.getObject()
##            if obj:
##                yield obj
##                # XXX safe my butt
##                obj._p_deactivate()
##
##class Wiki2PHCMigrator(CMFItemMigrator):
###class Wiki2PHCMigrator(BaseMigrator):
##    """
##    """
##    
##    fromType = 'Wiki Page'
##    toType   = 'HelpCenterHowTo'
##    #map = {'render' : 'setBody'}
##    
##    def __init__(self, obj, **kwargs):
##        BaseMigrator.__init__(self, obj, **kwargs)
##        self.new_id = mkNiceId(self.new_id)
##
##    def custom(self):
##        print '%s -> %s' % (self.old.absolute_url(1), self.new.absolute_url(1))
##        data = self.old.read()
##        self.new.setBody(data, encoding='utf-8', mimetype='text/structured')
##        self.new.setRelated_keywords((self.orig_id,))
##    
##    def renameOld(self):
##        """Renames the old object
##        
##        Unused
##        """
##        pass
##
##    def createNew(self):
##        """Create the new object
##        
##        use destination from kwargs
##        """
##        destination = self.kwargs['destination']
##        ttool = getToolByName(self.parent, 'portal_types')
##        typeInfo = ttool.getTypeInfo(self.toType)
##        typeInfo.constructInstance(destination, self.new_id)
##        self.new = getattr(destination, self.new_id)
##
##    def remove(self):
##        """Removes the old item
##        
##        abused to change the workflow state
##        """
##        wftool = getToolByName(self.parent, 'portal_workflow')
##        wftool.doActionFor(self.new, 'submit')
##        #if wftool.getInfoFor(self.old, 'review_state') in ('published', 'pending'):
##        #    wftool.doActionFor(self.old, 'retract')
##
##
##def mkNiceId(id):
##    """
##    """
##    assert(isinstance(id, str))
##    newId = []
##    for s in id:
##        if s.isupper():
##            newId.append('-'+s.lower())
##        else:
##            newId.append(s)
##    strid = ''.join(newId)
##    if strid.startswith('-'):
##        strid = strid[1:]
##    return strid
##

def metadataDefinitionToAttribute(self):
    """Modify Definitions which have definition in metadata to use it in
    attribute instead.
    """
    
    out = StringIO()
    
    catalog = getToolByName(self, 'portal_catalog')
    res = catalog.searchResults(portal_type = 'HelpCenterDefinition')
    
    for r in res:
        definition = r.getObject()
        base = aq_base(definition)
        
        if hasattr(base, '_md'):
            try:
                base.description = base._md['description']
                del base._md['descripion']
                definition._p_changed = 1
                print >> out, definition.absolute_url()
            except KeyError:
                pass

    return out.getvalue()


def makeAllPagesVisible(self):
    """Make all the tutorial pages visible"""
    catalog = getToolByName(self, 'portal_catalog')
    workflow = getToolByName(self, 'portal_workflow')
    pages = catalog.searchResults(portal_type = 'HelpCenterTutorialPage')
    
    out = StringIO()
    
    for p in pages:
        page = p.getObject()
        if workflow.getInfoFor(page, 'review_state') == 'hidden':
            print >>out, "Making page", page.absolute_url(), "visible"
            workflow.doActionFor(page, 'publish')
    
    return out.getvalue()
    
def setVersion(self):
    """Set the default version"""
    # XXX: Do we need/want to do this?
    pass

def migrate(self):
    
    out = StringIO()
    
    print >> out, "Wiki-to-PHC migration turned off!"
    # catalog = getToolByName(self, 'portal_catalog')
    # destination = self.documentation.howto2.howto
    #w = CatalogWalker(Wiki2PHCMigrator, catalog, destination=destination)
    # w = DocumentationWalker(Wiki2PHCMigrator, catalog)
    # return w.go( destination=destination)
    
    print >> out, "Migrating Definition description from metadata to attribute"
    print >> out, "You must PACK THE DATABASE after this operation for the"
    print >> out, "history tabs to work, since the historical objects are not"
    print >> out, "migrated"
    print >> out, metadataDefinitionToAttribute(self)
    
    print >> out, "\n"
    print >> out, "Publishing all tutorial pages (since we now have a workflow)"
    print >> out, "and this defaults to 'hidden'"
    print >> out, makeAllPagesVisible(self)
        
    return out.getvalue()



## Script (Python) "edit_reminder"
##title=Check edited object and print state-aware message.
##parameters=

wtool = context.portal_workflow

if wtool.getInfoFor(context, 'review_state') == 'in-progress':
    msg = 'Content+saved.%20+It+must+be+published+before+it+will+be+visible+to+others.%20+Please+submit+it+for+review+when+completed.'
else:
    msg='Your+changes+have+been+saved.'   

return context.REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (context.absolute_url(),msg) )



from zLOG import INFO
from Products.Archetypes.utils import OrderedDict
import Data

def CreateRootPHC( self, portal ):
    self.setRoles(('Manager',))
    portal.invokeFactory( 'HelpCenter', Data.Hc.Id)
    helpCenter = getattr( portal, Data.Hc.Id )
    helpCenter.setTitle(Data.Hc.Title)
    helpCenter.setDescription(Data.Hc.Desc)
    helpCenter.setVersionsVocab(Data.Hc.Versions)
    helpCenter.setSectionsVocab(Data.Hc.Sections)
    portal.portal_workflow.doActionFor( helpCenter, Data.Transition.publish )
    helpCenter.howto = getattr(helpCenter, "how-to")
    helpCenter.howto.sectionsVocab = Data.HowtoFolder.Sections
    helpCenter.tutorial.sectionsVocab = Data.TutorialFolder.Sections
    helpCenter.faq.sectionsVocab = Data.FAQFolder.Sections
    self.setRoles(('Member',))
    return "Created a PHC instance in the root of your Plone site."

def CreateAltPHC( self, portal ):
    self.setRoles(('Manager',))
    portal.invokeFactory( 'HelpCenter', Data.AltHc.Id)
    altHelpCenter = getattr( portal, Data.AltHc.Id )
    altHelpCenter.setTitle(Data.AltHc.Title)
    altHelpCenter.setDescription(Data.AltHc.Desc)
    altHelpCenter.setVersionsVocab(Data.AltHc.Versions)
    portal.portal_workflow.doActionFor( altHelpCenter, Data.Transition.publish )
    altHelpCenter.howto = getattr(altHelpCenter, "how-to")
    altHelpCenter.howto.sectionsVocab = Data.HowtoFolder.Sections
    altHelpCenter.tutorial.sectionsVocab = Data.TutorialFolder.Sections
    altHelpCenter.faq.sectionsVocab = Data.FAQFolder.Sections
    self.setRoles(('Member',))
    return "Created an alternate PHC instance in the root of your Plone site."

def CreateUsers( self, portal ):
    i = 0
    for user in Data.User.list:
        portal.portal_membership.addMember( user.Id, user.Password, user.Roles, [] )
        i += 1
    return "Created %d test users" % i

def CreateTutorials( self, portal ):
    self.setRoles(('Manager',))
    i = 0
    helpCenter = getattr( portal, Data.Hc.Id )
    for content in Data.Tutorial.list:
        helpCenter.tutorial.invokeFactory( 'HelpCenterTutorial', content.Id)
        newTutorial = getattr( helpCenter.tutorial, content.Id )
        newTutorial.setTitle(content.Title)
        newTutorial.setDescription(content.Summary)
        newTutorial.setVersions(content.Versions)
        newTutorial.setSections(content.Sections)
        # portal.plone_utils.changeOwnershipOf( newTutorial, content.Owner.Id, 1 )
        # Attach pages to the tutorial.
        for page in content.Pages:
            newTutorial.invokeFactory( 'HelpCenterTutorialPage', page.Id)
            newPage = getattr( newTutorial, page.Id )
            newPage.setTitle(page.Title)
            newPage.setDescription(page.Summary)
            newPage.setBody(page.Body)
            portal.plone_utils.editMetadata( newPage, format=page.Format )
            # Each page should be owned by the same owner as the tutorial owner
            # portal.plone_utils.changeOwnershipOf( newPage, content.Owner.Id, 1 )
            if page.Transition:
                portal.portal_workflow.doActionFor( newPage, page.Transition )
        # Update the tutorial's workflow state
        if content.Transition:
            portal.portal_workflow.doActionFor( newTutorial, content.Transition )

        i += 1
    self.setRoles(('Member',))
    return "Created %d PHC Tutorials." % i

def CreateHowtos( self, portal ):
    self.setRoles(('Manager',))
    i = 0
    helpCenter = getattr( portal, Data.Hc.Id )
    for content in Data.Howto.list:
        helpCenter.howto.invokeFactory( 'HelpCenterHowTo', content.Id)
        newHowto = getattr( helpCenter.howto, content.Id )
        newHowto.setTitle(content.Title)
        newHowto.setDescription(content.Summary)
        newHowto.setBody(content.Body)
        newHowto.setVersions(content.Versions)
        newHowto.setSections(content.Sections)
        portal.plone_utils.editMetadata( newHowto, format=content.Format )
        newHowto.reindexObject()
        # portal.plone_utils.changeOwnershipOf( newHowto, content.Owner.Id, 1 )
        if content.Transition:
            portal.portal_workflow.doActionFor( newHowto, content.Transition )
        i += 1
    return "Created %d PHC Howtos." % i

def CreateFaqs( self, portal, alt=False ):
    self.setRoles(('Manager',))
    i = 0
    if alt:
        helpCenter = getattr( portal, Data.AltHc.Id )
    else:
        helpCenter = getattr( portal, Data.Hc.Id )
    
    for content in Data.FAQ.list:
        helpCenter.faq.invokeFactory( 'HelpCenterFAQ', content.Id)
        newFaq = getattr( helpCenter.faq, content.Id )
        newFaq.setTitle(content.Title)
        newFaq.setDescription(content.Question)
        newFaq.setAnswer(content.Answer)
        newFaq.setVersions(content.Versions)
        newFaq.setSections(content.Sections)
        newFaq.reindexObject()
        #portal.plone_utils.editMetadata( newFaq, format=content.Format )
        #portal.plone_utils.changeOwnershipOf( newFaq, content.Owner.Id, 1 )
        if content.Transition:
            portal.portal_workflow.doActionFor( newFaq, content.Transition )
        i += 1
    self.setRoles(('Member',))
    return "Created %d PHC FAQs." % i


def CreateErrorRefs( self, portal ):
    i = 0
    return "Created %d PHC Error References." % i

def CreateDefinitions( self, portal ):
    i = 0
    return "Created %d PHC Definitions." % i

def CreateLinks( self, portal ):
    i = 0
    return "Created %d PHC Links." % i

def CreateReferenceManuals( self, portal ):
    i = 0
    return "Created %d PHC ReferenceManuals." % i

def CreateVideos( self, portal ):
    i = 0
    # See CMFPlone/tests/dummy.py for faking a FileField
    return "Created %d PHC Videos." % i

def CreateTestData( self, portal ):
    out = []
    out.append( CreateRootPHC( self, portal ) )
    out.append( CreateUsers( self, portal ) ) 
    out.append( CreateHowtos( self, portal ) )
    out.append( CreateTutorials( self, portal ) )
    out.append( CreateFaqs( self, portal ) )
    out.append( CreateErrorRefs( self, portal ) )
    out.append( CreateDefinitions( self, portal ) )
    out.append( CreateLinks( self, portal ) )
    out.append( CreateReferenceManuals( self, portal ) )
    out.append( CreateVideos( self, portal ) )
    return '  \n'.join( out )

functions = OrderedDict()
functions['Create Test Data'] = CreateTestData
functions['Create Test Users'] = CreateUsers
functions['Create Test PloneHelpCenter'] = CreateRootPHC
functions['Create Test Tutorials'] = CreateTutorials
functions['Create Test Howtos'] = CreateHowtos
functions['Create Test FAQs'] = CreateFaqs
functions['Create Test Error References'] = CreateReferenceManuals
functions['Create Test Definitions'] = CreateDefinitions
functions['Create Test Links'] = CreateLinks
functions['Create Test Reference Manuals'] = CreateReferenceManuals
functions['Create Test Videos'] = CreateVideos



""" support for HelpCenter container templates """

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.PythonScripts.standard import url_quote_plus


# Compare section brains by title
def _sectionCmp(a, b):
    # depends on cmp(True, False) == 1
    ash = getattr(a, 'getStartHere', False)
    bsh = getattr(b, 'getStartHere', False)
    if ash == bsh:
        return cmp(a.Title.lower(), b.Title.lower())
    elif ash:
        return -1
    else:
        return 1


class HelpCenterFolderView(BrowserView):
    """ support for HelpCenter container templates """
    
    def __init__(self, context, request):
        """ set up a few convenience object attributes """
        BrowserView.__init__(self, context, request)


    def Versions(self):
        """Method to display the versions in a nicer way."""
        return ", ".join(getattr(self.context, 'versions', []))


    def getItemsBySection(self, section, **kwargs):
        """Get items in this section"""
        
        context = self.context
        
        criteria = {}
        if section == 'No section':
            items = []
            for item in context.getFolderContents(contentFilter = kwargs):
                if not item.getSections:
                    items.append(item)
            return items
        else:
            criteria['getSections'] = section
            return context.getFolderContents(contentFilter = criteria)

    def getItemsBySections(self, **kwargs):
        """Get all items to list, by section only. Returns a list of dicts:

        'id'      : A normalised string representing the section
        'section' : The name of the section
        'items'   : A list of catalog brains for items in this section

        The first item will have an section title of 'No section' and contain
        all items with no section selected.
        """
        
        context = self.context
        
        plone_utils = getToolByName(context, 'plone_utils')
        brains = context.getFolderContents(contentFilter = kwargs)

        charset = context.getCharset()

        # Set up the dicts listing all sections
        # This is needed because we want it to list the audiences and sections
        # in the order the vocab specifies them.

        sections = []
        for s in ['No section'] + list(context.getSectionsVocab()):
            t = s.encode(charset)
            sections.append({'id'      : plone_utils.normalizeString(t),
                             'section' : t,
                             'items'   : []})

        # Then insert each how-to in the appropriate audience/section
        for b in brains:
            itemSections = b.getSections or ['No section']
            matchedSections = [s for s in sections if s['section'] in itemSections]
            for s in matchedSections:
                s['items'].append(b)

        # Finally clean out empty audiences or sections
        delSections = []
        for j in range(len(sections)):
            if len(sections[j]['items']) == 0:
                delSections.append(j)
        delSections.reverse()
        for j in delSections:
            del sections[j]

        # sort inside sections
        for j in sections:
            j['items'].sort(_sectionCmp)

        return sections


    def getItemsByAudiencesAndSections(self, **kwargs):
        """Get all items to list, by audience and section. Returns a list of
        dicts:

            'audience'  : The name of the audience category
            'sections'  : A list of dicts:

                'id'      : A normalised string representing the section
                'section' : The name of the section
                'items'   : A list of catalog brains for items in this section

        The first item will have an 'audience' title of 'Any audience' and 
        contain all items with no audience selected.
        """

        context = self.context

        plone_utils = getToolByName(context, 'plone_utils')
        brains = context.getFolderContents(contentFilter = kwargs)

        charset = context.getCharset()

        # Set up the dicts listing all audiences + sections
        # This is needed because we want it to list the audiences and sections
        # in the order the vocab specifies them.
        audiences = []
        for a in ['Any audience'] + list(context.getAudiencesVocab()):
            sections = []
            for s in ['No section'] + list(context.getSectionsVocab()):
                t = s.encode(charset)
                sections.append({'id'      : plone_utils.normalizeString(t),
                                 'section' : t,
                                 'items'   : []})
            audiences.append({'id'       : plone_utils.normalizeString(a),
                              'audience' : a,
                              'sections' : sections})

        # Then insert each how-to in the appropriate audience/section
        for b in brains:
            itemSections = [s for s in b.getSections if s]
            if not itemSections:
                itemSections = ['No section']
            else:
                # we don't want to display an item under both a major and minor
                # section. Presence of a colon indicates a subsection.
                for section in itemSections[:]:
                    cpos = section.find(':')
                    if cpos > 0:
                        try:
                            itemSections.remove(section[:cpos])
                        except ValueError:
                            pass

            itemAudiences = b.getAudiences or ['Any audience']            
            matchedAudiences = [a for a in audiences if a['audience'] in itemAudiences]
            if not matchedAudiences:
                # put it in 'Any audience'
                matchedAudiences = [audiences[0]]
            for a in matchedAudiences:
                matchedSections = [s for s in a['sections'] if s['section'] in itemSections]
                if not matchedSections:
                    # put it in 'No section'
                    matchedSections = [a['sections'][0]]
                for s in matchedSections:
                    s['items'].append(b)

        # Finally clean out empty audiences or sections
        delAudiences = []
        for i in range(len(audiences)):
            a = audiences[i]
            delSections = []
            for j in range(len(a['sections'])):
                if len(a['sections'][j]['items']) == 0:
                    delSections.append(j)
            delSections.reverse()
            for j in delSections:
                del a['sections'][j]
            if len(a['sections']) == 0:
                delAudiences.append(i)
        delAudiences.reverse()
        for i in delAudiences:
            del audiences[i]

        # sort inside sections            
        for a in audiences:
            for s in a['sections']:
                s['items'].sort(_sectionCmp)

        return audiences


    def getSectionsToList(self, **kwargs):
        """Sections that have at least one listable item. Note that this does 
        not take account of audiences.

        May return [] if there are no sections in the vocabulary
        """

        context = self.context

        sections = {}
        allSections = context.getSectionsVocab()

        if len(allSections) == 0:
            return []

        max_sections = len(allSections)

        for o in context.getFolderContents(contentFilter = kwargs):
            for s in o.getSections:
                sections[s]=1
            if len(sections) == max_sections:
                break
        return [s for s in allSections if sections.has_key(s)]



try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchema
from PHCContent import PHCContent

ReferenceManualPageSchema = HelpCenterBaseSchema + Schema((
    TextField(
        'description',
        default='',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                description="Enter a brief description.",
                description_msgid="phc_desc_referencemanual_page",
                label="Description",
                label_msgid="phc_label_referencemanual_page",
                rows=5,
                i18n_domain="plonehelpcenter",
                )
        ),

    TextField(
        'body',
        required=1,
        searchable=1,
        primary=1,
        widget=RichWidget(
                description = "The body text.",
                description_msgid = "phc_desc_body_referencemanual",
                label = "Body text",
                label_msgid = "phc_label_body_referencemanual",
                rows = 25,
                i18n_domain = "plonehelpcenter"
                ),
        **DEFAULT_CONTENT_TYPES
        )
    ),)

class HelpCenterReferenceManualPage(PHCContent,BaseContent):
    """Part of a reference manual."""

    __implements__ = (PHCContent.__implements__,
                      BaseContent.__implements__,)

    schema = ReferenceManualPageSchema
    archetype_name = 'Page'
    meta_type='HelpCenterReferenceManualPage'
    content_icon = 'document_icon.gif'

    global_allow = 0
    # allow_discussion = 1

    typeDescription= 'A Reference Manual Page contains the text of one of the pages of the the reference manual, usually confined to a single topic.'
    typeDescMsgId  = 'description_edit_referencemanualpage'


registerType(HelpCenterReferenceManualPage, PROJECTNAME)




try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.PloneHelpCenter.config import *
from Products.PloneHelpCenter.interfaces import IHelpCenterHowTo
from schemata import HelpCenterBaseSchemaFolderish, HelpCenterItemSchema
from PHCContent import PHCContent
from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder

from zope import event
from zope.interface import implements
try:
    from zope.lifecycleevent import ObjectModifiedEvent
except ImportError:
    from zope.app.event.objectevent import ObjectModifiedEvent


HowToSchema = HelpCenterBaseSchemaFolderish + Schema((
    TextField(
        'description',
        default='',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
            description = 'Brief explanation of the How To.',
            description_msgid = "phc_help_detailed_howto",
            label = "Summary",
            label_msgid = "phc_label_detailed_howto",
            i18n_domain = "plonehelpcenter",
            ),
        ),

    TextField(
        'body',
        searchable=1,
        required=1,
        primary=1,
        accessor='getBody',
        widget=RichWidget(
            description_msgid='phc_desc_howto_body',
            description='The text of the Howto',
            label_msgid='phc_body',
            label='Body',
            i18n_domain = "plonehelpcenter",
            rows=25,
            ),

        **DEFAULT_CONTENT_TYPES
        ),
    ),) + HelpCenterItemSchema

# For some reason, we need to jump through these hoops to get the fields in the
# the right order
HowToSchema.moveField('subject', pos='bottom')
HowToSchema.moveField('relatedItems', pos='bottom')

MAPPING = {'text_html' : 'text/html'}

# TODO: Generalize and i18nize
def addHelpCenterHowTo(self, id, **kwargs):
    obj = HelpCenterHowTo(id)
    self._setObject(id, obj)
    obj = self._getOb(id)
    obj.initializeArchetype(**kwargs)
    event.notify(ObjectModifiedEvent(obj))
    #setattr(obj, 'body', howto_template)
    # we need to get the template from the skin, so that it's customizable
    template = getattr(self, 'HowToTemplate', None)
    if template is not None:
        setattr(obj, 'body', template(self))
    return obj.getId()

class HelpCenterHowTo(PHCContent,BaseFolder):
    """A How-to is a document describing how to address a single, common 
    use-case or issue. You may add images and files as attachments.
    """
    
    implements(IHelpCenterHowTo)

    __implements__ = (PHCContent.__implements__,
                      BaseFolder.__implements__,
                      INonStructuralFolder
                      )

    content_icon = 'howto_icon.gif'

    typeDescription= 'A How-to is a document describing how to address a single, common use-case or issue. You may add images and files as attachments.'
    typeDescMsgId  = 'description_edit_howto'


    schema = HowToSchema
    archetype_name = 'How-to'
    meta_type = 'HelpCenterHowTo'
    global_allow = 0
    filter_content_types = 1
    # allow_discussion = IS_DISCUSSABLE
    allowed_content_types = ('Image', 'File',)
    default_view = 'howto_view'

    # actions = PHCContent.actions + (
    #     {
    #         'id': 'attachments',
    #         'name': 'Attachments',
    #         'action': 'string:${object_url}/attachments',
    #         'permissions': (CMFCorePermissions.ModifyPortalContent,)
    #         },
    # )
    # 
    # 
    # aliases = PHCContent.aliases.copy()
    # aliases.update({'(Default)'   : 'howto_view',
    #                 'view'        : 'howto_view',
    #                 'attachments' : 'phc_attachments'})

    security = ClassSecurityInfo()

    def getText(self):
        return 'nisse'

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setFormat')
    def setFormat(self, value):
        value = MAPPING.get(value, value)
        BaseFolder.setFormat(self, value)

registerType(HelpCenterHowTo, PROJECTNAME)
